//`include "svga_defines.v"

// DEFINE THE VARIOUS PIPELINE DELAYS

`define ZBT_PIPELINE_DELAY 	2
`define ZBT_INTERFACE_DELAY  	2
`define CHARACTER_DECODE_DELAY  4	

//  640 X 480 @ 60Hz with a 25.175MHz pixel clock
`define H_ACTIVE		640	// pixels
`define H_FRONT_PORCH	16	// pixels
`define H_SYNCH			96	// pixels
`define H_BACK_PORCH	48	// pixels
`define H_TOTAL			800	// pixels

`define V_ACTIVE		480	// lines
`define V_FRONT_PORCH	11	// lines
`define V_SYNCH			2	// lines
`define V_BACK_PORCH	31	// lines
`define V_TOTAL			524	// lines

`define W_HAT_TOP 15
`define W_HAT_BOTTOM 25
`define W_HEAD 15
`define W_NOSE 5
`define W_EYE 2
`define W_MUSTACHE 10
`define W_SHOULDER 25
`define W_ARM 29
`define W_BODY 17
`define W_HAND 6
`define W_TROUSER_1 25
`define W_LEG 10
`define W_FOOT 14
`define W_GOOMBA_HEAD 15
`define W_GOOMBA_HEAD2 19
`define W_GOOMBA_HEAD3 23
`define W_GOOMBA_FOOT 4
`define W_GOOMBA_EYE 3
`define W_PIPE_TOP 60
`define W_PIPE_BOTTOM 30

//begin mario module
module mario(
// connections to the DAC
VGA_OUT_PIXEL_CLOCK_P,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH_N,
VGA_VSYNCH_N,
VGA_OUT_RED_P,
VGA_OUT_GREEN_P,
VGA_OUT_BLUE_P,

// connections to the ZBT RAM
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P,

pixel_clock,
reset,

// connections to push buttons
pb_clock,
pb_data,
startup);

output 		VGA_OUT_PIXEL_CLOCK_P;	// pixel clock for the video DAC
output		VGA_COMP_SYNCH_N;	// composite sync for the video DAC
output 		VGA_OUT_BLANK_N;	// composite blanking for the video DAC
output 		VGA_HSYNCH_N;		// horizontal sync for the VGA output connector
output		VGA_VSYNCH_N;		// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED_P;		// RED DAC data
output [7:0] 	VGA_OUT_GREEN_P;	// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE_P;		// BLUE DAC data

output [18:0]	MEMORY_ADDR_P;		// external memory address bus
output 		MEMORY_CLK_P;		// external memory clock
output 		MEMORY_CLKEN_N;		// external memory clock enable
output 		MEMORY_WEN_N;		// external memory read/write
output 		MEMORY_WENA_N;		// external memory write enable for byte A
output 		MEMORY_WENB_N;		// external memory write enable for byte B
output 		MEMORY_WENC_N;		// external memory write enable for byte C
output 		MEMORY_WEND_N;		// external memory write enable for byte D
output 		MEMORY_OEN_N;		// external memory output enable
output 		MEMORY_CEN_N;		// external memory chip enable
output 		MEMORY_ADV_LD_N;	// external memory load/advance
inout [7:0] 	MEMORY_DATA_A_P;	// external memory byte A bits [ 7: 0] BLUE DATA
inout [7:0] 	MEMORY_DATA_B_P;	// external memory byte B bits [15: 8] GREEN DATA
inout [7:0] 	MEMORY_DATA_C_P;	// external memory byte C bits [23:16] RED DATA
inout [7:0] 	MEMORY_DATA_D_P;	// external memory byte D bits [31:24] MISC DATA


input	 		pixel_clock	/* synthesis syn_noclockbuf = 1 */; // pixel clock input 
input			reset;		// reset

input			pb_clock;	// push button clock
input			pb_data;	// push button data
output			startup;	// CPLD startup to initialize push buttons


wire			VGA_OUT_PIXEL_CLOCK_P;
wire 			VGA_COMP_SYNCH_N;
wire			VGA_OUT_BLANK_N;
wire			VGA_HSYNCH_N;
wire			VGA_VSYNCH_N;
wire	[7:0]		VGA_OUT_RED_P; 
wire	[7:0]		VGA_OUT_GREEN_P; 
wire	[7:0]		VGA_OUT_BLUE_P;


wire  	[18:0]		MEMORY_ADDR_P;
wire	 		MEMORY_CLK_P;
wire	 	 	MEMORY_CLKEN_N;
wire	 		MEMORY_WEN_N;
wire	 	 	MEMORY_WENA_N;
wire	 	 	MEMORY_WENB_N;
wire	 	 	MEMORY_WENC_N;
wire	 	 	MEMORY_WEND_N;
wire	 	 	MEMORY_OEN_N;
wire	 	 	MEMORY_CEN_N;
wire	 	 	MEMORY_ADV_LD_N;
wire	[7:0] 		MEMORY_DATA_A_P;
wire	[7:0] 		MEMORY_DATA_B_P;
wire	[7:0] 		MEMORY_DATA_C_P;
wire	[7:0] 		MEMORY_DATA_D_P;

reg 	[18:0]		user_vga_addr;
reg 			user_vga_read;
reg 			user_vga_write;
reg 			user_vga_write_byte_a;
reg 			user_vga_write_byte_b;
reg 			user_vga_write_byte_c;
reg 			user_vga_write_byte_d;
reg 			user_vga_enable;
wire 	[31:0]		user_vga_read_data;
reg  	[31:0]		user_vga_write_data;
wire			user_access_ok;
reg			user_clock_enable;
reg 			user_load_vga_addr;


//added variables
reg	[9:0]		line_count;		// counts the display lines
reg 	[10:0]		pixel_count;		// counts the pixels in a line	

reg 	[18:0]		address_count;		//address counter
reg 	[7:0]		frame_count;		//frame counter

//variables for drawing
reg 	[18:0]		mario_start_add;	//starting address where Mario is drawn
reg	[7:0]		mario_frame_count;	//Mario frame count
reg	[18:0]		goomba_start_add;	//starting address where goomba is drawn
reg	[7:0]		goomba_frame_count;	//goomba frame count

reg  			jump;			//variable to determine if jump occurred
reg 	[5:0]		jump_count;		//count how long Mario has jumped for

wire 			pixel_clock;
wire 			reset;

wire			pb_clock;
wire			pb_data;
wire			startup;

assign startup = 1'b1;				//initialize push buttons

always @ (posedge reset or posedge pb_clock) begin
	if (reset) begin
		jump <= 1'b0;
		jump_count <= 6'd0;
	end else if (pb_data == 1'b1) begin 	//Mario jumps if push button is pressed
		jump <= 1'b1;
		jump_count <= 6'd0;
		end
	end

// CREATE THE HORIZONTAL LINE PIXEL COUNTER
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin					// on reset set pixel counter to 0
			pixel_count <= 11'h000;
		end

	else if (pixel_count == (`H_TOTAL - 1))		// last pixel in the line
 		begin					
			pixel_count <= 11'h000;		// reset pixel counter
		end

	else 	begin
			pixel_count <= pixel_count +1;
		end
	end

// CREATE THE VERTICAL FRAME LINE COUNTER
always @ (posedge pixel_clock or posedge reset) begin
	if (frame_count == 8'h3C)
		begin
			frame_count <= 8'h00;		//reset frame when it equals 60
		end

	if (reset)
 		begin					// on reset set line counter to 0
			line_count <= 10'h000;
		end

	else if ((line_count == (`V_TOTAL - 1))&& (pixel_count == (`H_TOTAL - 1)))
		begin					// last pixel in last line of frame 
			line_count <= 10'h000;		// reset line counter

			frame_count <= frame_count + 8'h01; //increment frame after frame is drawn
		end

	else if ((pixel_count == (`H_TOTAL - 1)))
		begin						// last pixel but not last line
			line_count <= line_count + 1;	// increment line counter
		end
	end

//begin mario display
always @ (posedge pixel_clock) begin

	//increment address
	if (user_access_ok) begin	
		user_vga_addr <= address_count;
		address_count <= address_count + 1;
		end

	//reset address count when it reaches 640x480
	if (address_count == 19'h4B000) begin
		address_count <= 19'h00000;
		user_vga_write_data <= 32'h00000000;
		end

	//goomba movement
	if (frame_count == goomba_frame_count + 8'd10 && goomba_dead == 1'b0)
		begin
			goomba_frame_count <= frame_count;
			goomba_start_add <= goomba_start_add - 4'd5;
		end

	//reset goomba after it leaves screen
	if (goomba_start_add <= 19'd240640)
		begin
			goomba_start_add <= 19'd241280;
		end

	//reset goomba frame count once it reaches 60
	if (goomba_frame_count >= 8'd60)
		begin
			goomba_frame_count <= 8'd00;
		end

	//mario movement
	if (frame_count == mario_frame_count + 8'd10)
		begin
			mario_frame_count <= frame_count;
			mario_start_add <= mario_start_add + 4'd5;

			//make mario fly!
			if (jump == 1'b1 && jump_count < 4'd4) begin
				mario_start_add <= mario_start_add - 19'd3190;
				jump_count <= jump_count + 4'd1;
			end else if (jump == 1'b1 && jump_count >= 4'd4 && jump_count < 4'd8) begin
				mario_start_add <= mario_start_add + 19'd3210;
				jump_count <= jump_count + 4'd1;
			end
		end

	//reset mario frame count once it reaches 60
	if (mario_frame_count >= 8'd60)
		begin
			mario_frame_count <= 8'd00;
		end

	//reset position of marion once he leaves screen
	if (mario_start_add >= 19'd225280)
		begin
			mario_start_add <= 19'd224640;
		end	

	//draw from line 0 to line 400 (640x400 = 256000)
	if (address_count < 19'd256000)
		begin
			//draw skin of mario
			if ((address_count >= mario_start_add + 19'd4480 && address_count <= mario_start_add + 19'd4480 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd5120 && address_count <= mario_start_add + 19'd5120 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd5760 && address_count <= mario_start_add + 19'd5760 + `W_HEAD - 19'd7) ||
				(address_count >= mario_start_add + 19'd5760 + `W_HEAD - 19'd5 && address_count <= mario_start_add + 19'd5760 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd6400 && address_count <= mario_start_add + 19'd6400 + `W_HEAD - 19'd7) ||
				(address_count >= mario_start_add + 19'd6400 + `W_HEAD - 19'd5 && address_count <= mario_start_add + 19'd6400 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd7040 && address_count <= mario_start_add + 19'd7040 + `W_HEAD + `W_NOSE - 19'd1) ||
				(address_count >= mario_start_add + 19'd7680 && address_count <= mario_start_add + 19'd7680 + `W_HEAD + `W_NOSE - 19'd1) ||
				(address_count >= mario_start_add + 19'd8320 && address_count <= mario_start_add + 19'd8320 + `W_HEAD + `W_NOSE) ||
				(address_count >= mario_start_add + 19'd8960 && address_count <= mario_start_add + 19'd8960 + `W_HEAD + `W_NOSE) ||
				(address_count >= mario_start_add + 19'd9600 && address_count <= mario_start_add + 19'd9600 + `W_HEAD + `W_NOSE) ||
				(address_count >= mario_start_add + 19'd10240 && address_count <= mario_start_add + 19'd10240 + `W_HEAD + `W_NOSE) ||
				(address_count >= mario_start_add + 19'd10880 && address_count <= mario_start_add + 19'd10880 + `W_HEAD + `W_NOSE - 19'd1 - `W_MUSTACHE) ||
				(address_count >= mario_start_add + 19'd11520 && address_count <= mario_start_add + 19'd11520 + `W_HEAD + `W_NOSE - 19'd1 - `W_MUSTACHE) ||
				(address_count >= mario_start_add + 19'd12160 && address_count <= mario_start_add + 19'd12160 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd12800 && address_count <= mario_start_add + 19'd12800 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd13440 && address_count <= mario_start_add + 19'd13440 + `W_HEAD) ||
				(address_count >= mario_start_add + 19'd14080 && address_count <= mario_start_add + 19'd14080 + `W_HEAD) ||  						
				//9 lines from head to arm
				(address_count >= mario_start_add + 19'd19834 && address_count <= mario_start_add + 19'd19834 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd19834 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd19834 + (2*`W_HAND) + `W_BODY) ||
				(address_count >= mario_start_add + 19'd20474 && address_count <= mario_start_add + 19'd20474 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd20474 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd20474 + (2*`W_HAND) + `W_BODY) ||
				(address_count >= mario_start_add + 19'd21114 && address_count <= mario_start_add + 19'd21114 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd21114 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd21114 + (2*`W_HAND) + `W_BODY) ||
				(address_count >= mario_start_add + 19'd21754 && address_count <= mario_start_add + 19'd21754 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd21754 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd21754 + (2*`W_HAND) + `W_BODY) ||
				(address_count >= mario_start_add + 19'd22394 && address_count <= mario_start_add + 19'd22394 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd22394 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd22394 + (2*`W_HAND) + `W_BODY) ||
				(address_count >= mario_start_add + 19'd23034 && address_count <= mario_start_add + 19'd23034 + `W_HAND) ||
				(address_count >= mario_start_add + 19'd23034 + `W_HAND + `W_BODY && address_count <= mario_start_add + 19'd23034 + (2*`W_HAND) + `W_BODY)
				)
				begin
					user_vga_write_data <= 32'h00FFCCCC;
		    		end
			//draw hat
			else if ((address_count >= mario_start_add && address_count <= mario_start_add + `W_HAT_TOP) ||
		    		(address_count >= mario_start_add + 19'd640 && address_count <= mario_start_add + 19'd640 + `W_HAT_TOP) ||
				(address_count >= mario_start_add + 19'd1280 && address_count <= mario_start_add + 19'd1280 + `W_HAT_TOP) ||
				(address_count >= mario_start_add + 19'd1920 && address_count <= mario_start_add + 19'd1920 + `W_HAT_TOP) ||
				(address_count >= mario_start_add + 19'd2560 && address_count <= mario_start_add + 19'd2560 + `W_HAT_BOTTOM) ||
				(address_count >= mario_start_add + 19'd3200 && address_count <= mario_start_add + 19'd3200 + `W_HAT_BOTTOM) ||
				(address_count >= mario_start_add + 19'd3840 && address_count <= mario_start_add + 19'd3840 + `W_HAT_BOTTOM) ||
				//start body
				(address_count >= mario_start_add + 19'd14715 && address_count <= mario_start_add + 19'd14715 + `W_SHOULDER) ||
				(address_count >= mario_start_add + 19'd15355 && address_count <= mario_start_add + 19'd15355 + `W_SHOULDER) ||
				(address_count >= mario_start_add + 19'd15995 && address_count <= mario_start_add + 19'd15995 + `W_SHOULDER) ||
				(address_count >= mario_start_add + 19'd16635 && address_count <= mario_start_add + 19'd16635 + `W_SHOULDER) ||
				(address_count >= mario_start_add + 19'd17275 && address_count <= mario_start_add + 19'd17275 + `W_SHOULDER) ||				
				(address_count >= mario_start_add + 19'd17914 && address_count <= mario_start_add + 19'd17914 + `W_ARM) ||
				(address_count >= mario_start_add + 19'd18554 && address_count <= mario_start_add + 19'd18554 + `W_ARM) ||
				(address_count >= mario_start_add + 19'd19194 && address_count <= mario_start_add + 19'd19194 + `W_ARM) ||
				(address_count >= mario_start_add + 19'd19840 && address_count <= mario_start_add + 19'd19840 + `W_BODY) ||
				(address_count >= mario_start_add + 19'd20480 && address_count <= mario_start_add + 19'd20480 + `W_BODY) ||
				(address_count >= mario_start_add + 19'd21120 && address_count <= mario_start_add + 19'd21120 + `W_BODY) ||
				(address_count >= mario_start_add + 19'd21760 && address_count <= mario_start_add + 19'd21760 + `W_BODY) ||
				(address_count >= mario_start_add + 19'd22400 && address_count <= mario_start_add + 19'd22400 + `W_BODY) ||
				(address_count >= mario_start_add + 19'd23040 && address_count <= mario_start_add + 19'd23040 + `W_BODY)
				)
				begin
					user_vga_write_data <= 32'h00DD0000;
		    		end
			else if ((address_count >= mario_start_add + 19'd5760 + `W_HEAD - 19'd7 && address_count <= mario_start_add + 19'd5760 + `W_HEAD - 19'd5) ||
				(address_count >= mario_start_add + 19'd6400 + `W_HEAD - 19'd7 && address_count <= mario_start_add + 19'd6400 + `W_HEAD - 19'd5) ||
				(address_count >= mario_start_add + 19'd10880 + `W_HEAD + `W_NOSE - 19'd1 - `W_MUSTACHE && address_count <= mario_start_add + 19'd10880 + `W_HEAD + `W_NOSE - 19'd1) ||
				(address_count >= mario_start_add + 19'd11520 + `W_HEAD + `W_NOSE - 19'd1 - `W_MUSTACHE && address_count <= mario_start_add + 19'd11520 + `W_HEAD + `W_NOSE - 19'd1)
				)
				begin
					user_vga_write_data <= 32'h00DD0000;
		    		end
		    	//draw legs for first 30 frames
			else if (frame_count < 8'h1E &&
				//start leg
				((address_count >= mario_start_add + 19'd23679 && address_count <= mario_start_add + 19'd23679 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd23689 && address_count <= mario_start_add + 19'd23689 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24319 && address_count <= mario_start_add + 19'd24319 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24329 && address_count <= mario_start_add + 19'd24329 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24959 && address_count <= mario_start_add + 19'd24959 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24969 && address_count <= mario_start_add + 19'd24969 + `W_LEG) ||
				//start leg shift + 1
				(address_count >= mario_start_add + 19'd25597 && address_count <= mario_start_add + 19'd25597 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd25611 && address_count <= mario_start_add + 19'd25611 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26237 && address_count <= mario_start_add + 19'd26237 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26251 && address_count <= mario_start_add + 19'd26251 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26877 && address_count <= mario_start_add + 19'd26877 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26891 && address_count <= mario_start_add + 19'd26891 + `W_LEG) ||
				//start leg shift + 2
				(address_count >= mario_start_add + 19'd27515 && address_count <= mario_start_add + 19'd27515 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd27533 && address_count <= mario_start_add + 19'd27533 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28155 && address_count <= mario_start_add + 19'd28155 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28173 && address_count <= mario_start_add + 19'd28173 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28795 && address_count <= mario_start_add + 19'd28795 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28813 && address_count <= mario_start_add + 19'd28813 + `W_LEG)
				))
				begin
					user_vga_write_data <= 32'h00DD0000;
				end
		    	//draw shoes for first 30 frames
			else if (frame_count < 8'h1E &&
				//start shoe
				((address_count >= mario_start_add + 19'd29435 && address_count <= mario_start_add + 19'd29435 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd29453 && address_count <= mario_start_add + 19'd29453 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30075 && address_count <= mario_start_add + 19'd30075 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30093 && address_count <= mario_start_add + 19'd30093 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30715 && address_count <= mario_start_add + 19'd30715 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30733 && address_count <= mario_start_add + 19'd30733 + `W_LEG)
				))
				begin
					user_vga_write_data <= 32'h00990000;
				end
		    	//draw legs for second 30 frames
			else if (frame_count >= 8'h1E &&
				//start leg
				((address_count >= mario_start_add + 19'd23675 && address_count <= mario_start_add + 19'd23675 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd23693 && address_count <= mario_start_add + 19'd23693 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24315 && address_count <= mario_start_add + 19'd24315 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24333 && address_count <= mario_start_add + 19'd24333 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24955 && address_count <= mario_start_add + 19'd24955 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd24973 && address_count <= mario_start_add + 19'd24973 + `W_LEG) ||
				//start leg shift + 1
				(address_count >= mario_start_add + 19'd25593 && address_count <= mario_start_add + 19'd25593 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd25615 && address_count <= mario_start_add + 19'd25615 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26233 && address_count <= mario_start_add + 19'd26233 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26255 && address_count <= mario_start_add + 19'd26255 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26873 && address_count <= mario_start_add + 19'd26873 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd26895 && address_count <= mario_start_add + 19'd26895 + `W_LEG) ||
				//start leg shift + 2
				(address_count >= mario_start_add + 19'd27511 && address_count <= mario_start_add + 19'd27511 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd27537 && address_count <= mario_start_add + 19'd27537 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28151 && address_count <= mario_start_add + 19'd28151 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28177 && address_count <= mario_start_add + 19'd28177 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28791 && address_count <= mario_start_add + 19'd28791 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd28817 && address_count <= mario_start_add + 19'd28817 + `W_LEG)
				))
				begin
					user_vga_write_data <= 32'h00DD0000;
		    		end
			//draw shoes for second 30 frames
			else if (frame_count >= 8'h1E &&
				//start shoe
				((address_count >= mario_start_add + 19'd29431 && address_count <= mario_start_add + 19'd29431 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd29457 && address_count <= mario_start_add + 19'd29457 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30071 && address_count <= mario_start_add + 19'd30071 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30097 && address_count <= mario_start_add + 19'd30097 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30711 && address_count <= mario_start_add + 19'd30711 + `W_LEG) ||
				(address_count >= mario_start_add + 19'd30737 && address_count <= mario_start_add + 19'd30737 + `W_LEG)
				))
				begin
					user_vga_write_data <= 32'h00990000;
				end
			//draw goomba (24 lines high)
			else if (goomba_dead == 1'b0 &&
				(
				(address_count >= goomba_start_add + 19'd10 && address_count <= goomba_start_add + 19'd10 + `W_GOOMBA_HEAD) ||
				(address_count >= goomba_start_add + 19'd650 && address_count <= goomba_start_add + 19'd650 + `W_GOOMBA_HEAD) ||
				(address_count >= goomba_start_add + 19'd1290 && address_count <= goomba_start_add + 19'd1290 + `W_GOOMBA_HEAD) ||
				(address_count >= goomba_start_add + 19'd1930 && address_count <= goomba_start_add + 19'd1930 + `W_GOOMBA_HEAD) ||
				(address_count >= goomba_start_add + 19'd2570 && address_count <= goomba_start_add + 19'd2570 + `W_GOOMBA_HEAD) ||
				//goomba head + eye
				(address_count >= goomba_start_add + 19'd3208 && address_count <= goomba_start_add + 19'd3208 + 19'd5) ||
				(address_count >= goomba_start_add + 19'd3208 + 19'd8 && address_count <= goomba_start_add + 19'd3208 + 19'd11) ||
				(address_count >= goomba_start_add + 19'd3208 + 19'd14 && address_count <= goomba_start_add + 19'd3208 + 19'd19) ||
				(address_count >= goomba_start_add + 19'd3848 && address_count <= goomba_start_add + 19'd3848 + 19'd5) ||
				(address_count >= goomba_start_add + 19'd3848 + 19'd8 && address_count <= goomba_start_add + 19'd3848 + 19'd11) ||
				(address_count >= goomba_start_add + 19'd3848 + 19'd14 && address_count <= goomba_start_add + 19'd3848 + 19'd19) ||
				(address_count >= goomba_start_add + 19'd4488 && address_count <= goomba_start_add + 19'd4488 + 19'd5) ||
				(address_count >= goomba_start_add + 19'd4488 + 19'd8 && address_count <= goomba_start_add + 19'd4488 + 19'd11) ||
				(address_count >= goomba_start_add + 19'd4488 + 19'd14 && address_count <= goomba_start_add + 19'd4488 + 19'd19) ||
				(address_count >= goomba_start_add + 19'd5128 && address_count <= goomba_start_add + 19'd5128 + `W_GOOMBA_HEAD2) ||
				(address_count >= goomba_start_add + 19'd5768 && address_count <= goomba_start_add + 19'd5768 + `W_GOOMBA_HEAD2) ||
				(address_count >= goomba_start_add + 19'd6406 && address_count <= goomba_start_add + 19'd6406 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd7046 && address_count <= goomba_start_add + 19'd7046 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd7686 && address_count <= goomba_start_add + 19'd7686 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd8326 && address_count <= goomba_start_add + 19'd8326 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd8966 && address_count <= goomba_start_add + 19'd8966 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd9606 && address_count <= goomba_start_add + 19'd9606 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd10246 && address_count <= goomba_start_add + 19'd10246 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd10886 && address_count <= goomba_start_add + 19'd10886 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd11526 && address_count <= goomba_start_add + 19'd11526 + `W_GOOMBA_HEAD3) ||
				(address_count >= goomba_start_add + 19'd12166 && address_count <= goomba_start_add + 19'd12166 + `W_GOOMBA_HEAD3))
				)
				begin
					user_vga_write_data <= 32'h00993300;
		    		end
			//goomba eye
			else if (goomba_dead == 1'b0 &&
				(
				(address_count >= goomba_start_add + 19'd3208 + 19'd5 && address_count <= goomba_start_add + 19'd3208 + 19'd8) ||
				(address_count >= goomba_start_add + 19'd3208 + 19'd11 && address_count <= goomba_start_add + 19'd3208 + 19'd14) ||
				(address_count >= goomba_start_add + 19'd3848 + 19'd5 && address_count <= goomba_start_add + 19'd3848 + 19'd8) ||
				(address_count >= goomba_start_add + 19'd3848 + 19'd11 && address_count <= goomba_start_add + 19'd3848 + 19'd14) ||
				(address_count >= goomba_start_add + 19'd4488 + 19'd5 && address_count <= goomba_start_add + 19'd4488 + 19'd8) ||
				(address_count >= goomba_start_add + 19'd4488 + 19'd11 && address_count <= goomba_start_add + 19'd4488 + 19'd14)
				))
				begin
					user_vga_write_data <= 32'h00FFFFFF;
		    		end
			//goomba feet for first 30 frames
			else if (frame_count < 8'd30 && goomba_dead == 1'b0 &&
				((address_count >= goomba_start_add + 19'd12810 && address_count <= goomba_start_add + 19'd12810 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd12821 && address_count <= goomba_start_add + 19'd12821 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd13450 && address_count <= goomba_start_add + 19'd13450 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd13461 && address_count <= goomba_start_add + 19'd13461 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14090 && address_count <= goomba_start_add + 19'd14090 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14101 && address_count <= goomba_start_add + 19'd14101 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14730 && address_count <= goomba_start_add + 19'd14730 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14741 && address_count <= goomba_start_add + 19'd14741 + `W_GOOMBA_FOOT)
				))
				begin
					user_vga_write_data <= 32'h00000000;
		    		end
			//goomba feet for second 30 frames
			else if (frame_count >= 8'd30 && goomba_dead == 1'b0 &&
				((address_count >= goomba_start_add + 19'd12806 && address_count <= goomba_start_add + 19'd12806 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd12825 && address_count <= goomba_start_add + 19'd12825 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd13446 && address_count <= goomba_start_add + 19'd13446 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd13465 && address_count <= goomba_start_add + 19'd13465 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14086 && address_count <= goomba_start_add + 19'd14086 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14105 && address_count <= goomba_start_add + 19'd14105 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14726 && address_count <= goomba_start_add + 19'd14726 + `W_GOOMBA_FOOT) ||
				(address_count >= goomba_start_add + 19'd14745 && address_count <= goomba_start_add + 19'd14745 + `W_GOOMBA_FOOT)
				))
				begin
					user_vga_write_data <= 32'h00000000;
		    		end
		    	//draw sky
			else
				begin
					user_vga_write_data <= 32'h000000F0;
				end
		end
	//ground
	else 
		begin
			user_vga_write_data <= 32'h00F00F00;
		end
	
//Memory control signals
always @ (posedge pixel_clock) begin
		
	if (user_access_ok) begin
		user_vga_read <= 1'b0;
		user_vga_write <= 1'b1;
		user_vga_write_byte_a <= 1'b1;
		user_vga_write_byte_b <= 1'b1;
		user_vga_write_byte_c <= 1'b1;
		user_vga_write_byte_d <= 1'b0;
		user_vga_enable <= 1'b1;
		user_clock_enable <= 1'b1;
		user_load_vga_addr <= 1'b1;
		end

end


BM_MODE_SVGA_CTRL BM_MODE_SVGA_CTRL_inst(
// connections to the DAC
.VGA_OUT_PIXEL_CLOCK_P(VGA_OUT_PIXEL_CLOCK_P),
.VGA_COMP_SYNCH_N(VGA_COMP_SYNCH_N),
.VGA_OUT_BLANK_N(VGA_OUT_BLANK_N),
.VGA_HSYNCH_N(VGA_HSYNCH_N),
.VGA_VSYNCH_N(VGA_VSYNCH_N),
.VGA_OUT_RED_P(VGA_OUT_RED_P),
.VGA_OUT_GREEN_P(VGA_OUT_GREEN_P),
.VGA_OUT_BLUE_P(VGA_OUT_BLUE_P),

// connections to the ZBT RAM
.MEMORY_CLK_P(MEMORY_CLK_P),
.MEMORY_CLKEN_N(MEMORY_CLKEN_N),
.MEMORY_WEN_N(MEMORY_WEN_N),
.MEMORY_WENA_N(MEMORY_WENA_N),
.MEMORY_WENB_N(MEMORY_WENB_N),
.MEMORY_WENC_N(MEMORY_WENC_N),
.MEMORY_WEND_N(MEMORY_WEND_N),
.MEMORY_ADV_LD_N(MEMORY_ADV_LD_N),
.MEMORY_OEN_N(MEMORY_OEN_N),
.MEMORY_CEN_N(MEMORY_CEN_N),
.MEMORY_ADDR_P(MEMORY_ADDR_P),
.MEMORY_DATA_A_P(MEMORY_DATA_A_P),
.MEMORY_DATA_B_P(MEMORY_DATA_B_P),
.MEMORY_DATA_C_P(MEMORY_DATA_C_P),
.MEMORY_DATA_D_P(MEMORY_DATA_D_P),

// connections to the user interface
.user_vga_addr(user_vga_addr),
.user_vga_read(user_vga_read),
.user_vga_write(user_vga_write),
.user_vga_write_byte_a(user_vga_write_byte_a),
.user_vga_write_byte_b(user_vga_write_byte_b),
.user_vga_write_byte_c(user_vga_write_byte_c),
.user_vga_write_byte_d(user_vga_write_byte_d),
.user_vga_enable(user_vga_enable),
.user_vga_read_data(user_vga_read_data),
.user_vga_write_data(user_vga_write_data),
.user_access_ok(user_access_ok),
.user_clock_enable(user_clock_enable),
.user_load_vga_addr(user_load_vga_addr),
.pixel_clock(pixel_clock),
.reset(reset)
);

endmodule